clear; close all
T = 0;
y_shift = 1;
y_scale = 5;
sample_num = 1e5;

q = 0.5;
p = [q 1-2*q q];

max_n = 200;
n = 0:max_n;

inf_max = 100;

% max_z = 100;
% waiting_times = 0:max_z;

fname_penalty = 'penalty_binary';
fname_penalty_sum = 'penalty_sum';

fname_expected_penalty = 'expected_penalty_discrete';
fname_expected_sum = 'expected_sum_discrete';

dist_matrix = p;
possible_y = [y_shift y_shift+y_scale y_shift+2*y_scale];
parameters = [possible_y; dist_matrix];

bournoulli_q_mean = 0:0.005:0.5;

[~,samples] = histc(rand(1,sample_num+1),cumsum(p(:))/sum(p)); % discrete distributions of 0,1,2%round(LogNormalMarkovSamples(alpha,mu, sigma,sample_num+1)/mean_tmp);
samples = samples'*y_scale+y_shift;
mean_ = mean(samples);

age_uniform_wait=zeros(1,length(bournoulli_q_mean));
age_zero_wait=zeros(1,length(bournoulli_q_mean));
age_optimal=zeros(1,length(bournoulli_q_mean));
wait_times_uniform = zeros(1, sample_num);
wait_times_zero = zeros(1, sample_num);
wait_times_optimal = zeros(1, sample_num);
k = 1; %Infinite for first point anyway
for coefficient = bournoulli_q_mean(2:end)
    coefficient
    k = k + 1;
    
    age_penalty_lut = feval(fname_penalty, n, coefficient);
    expected_sum_lut = expected_sum_generate_lut(age_penalty_lut, max_n);
%     age_sums_lut = feval(fname_sum, age_penalty_lut, samples, waiting_times);%, parameters);
%     expected_age_lut = mean(age_penalty_lut(samples(1:sample_num)+waiting_times+samples(2:sample_num+1)));
%   expected_age_lut2 = mean(feval(fname_age, age_penalty_lut, samples(1:sample_num), waiting_times, parameters));
%     expected_age_sum_lut = feval(fname_sum, age_penalty_lut, samples(1:sample_num), waiting_times, parameters);
    
    %% Uniform Wait  
    wait_times_uniform = ceil(mean_)*ones(1, sample_num);
    S_i = [0 cumsum(wait_times_uniform)];
    D_i = zeros(1,sample_num+1);
    D_i(1) = S_i(1)+samples(1);
    for i = 1:sample_num
        %Need the extra computation to account for the queueing delay.
        D_i(i+1) = max(S_i(i+1), D_i(i)) + samples(i+1);
    end
    Y_i = D_i(1:sample_num) - S_i(1:sample_num);
    Z_plus_Y_ip1 = D_i(2:sample_num+1) - D_i(1:sample_num);
    Expectedq = mean(feval(fname_penalty_sum, expected_sum_lut, Y_i, Z_plus_Y_ip1));
    age_uniform_wait(k) = Expectedq/(mean(D_i(2:sample_num+1) - D_i(1:sample_num)));
    
    %% Zero Wait
    if(T < mean_)
        S_i = [0 cumsum(samples(1:sample_num)')];
        D_i = cumsum(samples');
        Y_i = D_i(1:sample_num) - S_i(1:sample_num);
        Z_plus_Y_ip1 = D_i(2:sample_num+1) - D_i(1:sample_num);
        Expectedq = mean(feval(fname_penalty_sum, expected_sum_lut, Y_i, Z_plus_Y_ip1));
        age_zero_wait(k) = Expectedq/(mean(D_i(2:sample_num+1) - D_i(1:sample_num))); %E[sum(p(n-Si))]/E[Di+1-Di]
    end
    
    %% Optimal Policy
    optimal_policy_mutual_info;
    S_i = [0 cumsum(wait_times_optimal+samples(1:sample_num)')];
    D_i = zeros(1,sample_num+1);
    D_i(1) = S_i(1)+samples(1);
    for i = 1:sample_num
        %Need the extra computation to account for the queueing delay.
        D_i(i+1) = max(S_i(i+1), D_i(i)) + samples(i+1);
    end
    Y_i = D_i(1:sample_num) - S_i(1:sample_num);
    Z_plus_Y_ip1 = D_i(2:sample_num+1) - D_i(1:sample_num);
    Expectedq = mean(feval(fname_penalty_sum, expected_sum_lut, Y_i, Z_plus_Y_ip1));
    age_optimal(k) = Expectedq/(mean(D_i(2:sample_num+1) - D_i(1:sample_num))); %E[sum(p(n-Si))]/E[Di+1-Di]

end
color_index = 1;
age_uniform_wait(1) = -inf_max;
plot(bournoulli_q_mean,-age_uniform_wait,'b-.','linewidth',2);
hold on;
legendInfo{color_index} = 'Uniform Sampling'; 

color_index = color_index + 1;
age_zero_wait(1) = -inf_max;
plot(bournoulli_q_mean,-age_zero_wait,'r--','linewidth',2);
hold on;
legendInfo{color_index} = 'Zero Wait';

color_index = color_index + 1;
age_optimal(1) = -inf_max;
plot(bournoulli_q_mean,-age_optimal,'k','linewidth',2);
hold on;
legendInfo{color_index} = 'Optimal policy'; 

axis([0 0.5 0 0.6])
xlabel('Mean of Bernoulli Random Variables in Binary Markov Source (q)', 'FontSize', 16);
ylabel('Average Mutual Information', 'FontSize', 16);
legend(legendInfo, 'FontSize', 16);
saveas(figure(1),'figs/figure3_MutualInfo_T2_binary_discrete_test.fig');
